// -*- C++ -*-
#include "Rivet/Analysis.hh"
#include "Rivet/Projections/Beam.hh"
#include "Rivet/Projections/FinalState.hh"

namespace Rivet {


  /// @brief e+ e- > J/psi D Dbar
  class BELLE_2017_I1590028 : public Analysis {
  public:

    /// Constructor
    RIVET_DEFAULT_ANALYSIS_CTOR(BELLE_2017_I1590028);


    /// @name Analysis methods
    /// @{

    /// Book histograms and initialise projections before the run
    void init() {
      // set the PDG code
      _pid = getOption<int>("PID", 9010441);
      // projections
      declare(Beam(), "Beams");
      declare(FinalState(), "FS");
      if(isCompatibleWithSqrtS(10.58,1e-4)) {
	// histograms
	for(unsigned int ix=0;ix<6;++ix) {
	  book(_h[ix],1,1,1+ix);
	}
      }
      book(_c,"TMP/nEvent");
    }


    /// Perform the per-event analysis
    void analyze(const Event& event) {
      // get the axis, direction of incoming electron
      const ParticlePair& beams = apply<Beam>(event, "Beams").beams();
      Vector3 axis;
      FourMomentum pep;
      if(beams.first.pid()>0) {
	axis = beams.first .momentum().p3().unit();
	pep  = beams.second.momentum();
      }
      else {
	axis = beams.second.momentum().p3().unit();
	pep  = beams.first .momentum();
      }
      Particles fs = apply<FinalState>(event, "FS").particles();
      Particles DD,JPSI,other;
      for(const Particle & p : fs) {
	Particle parent=p;
	while(!parent.parents().empty()) {
	  parent=parent.parents()[0];
	  if(parent.abspid()==PID::DPLUS ||
	     parent.abspid()==PID::D0||
	     parent.pid()   ==PID::JPSI) break;
	}
	if(parent.abspid()!=PID::DPLUS &&
	   parent.abspid()!=PID::D0    &&
	   parent.pid()   !=PID::JPSI) {
	  other.push_back(p);
	  continue;
	}
	bool found=false;
	for (auto & D : parent.pid()==PID::JPSI ? JPSI : DD) {
	  // D or J/psi already in list 
	  if (fuzzyEquals(D.momentum(),parent.momentum())) {
	    found=true;
	    break;
	  }
	}
	if(!found) {
	  (parent.pid()==PID::JPSI ? JPSI : DD).push_back(parent);
	}
      }
      // require J/psi D Dbar
      if(JPSI.size()!=1 || DD.size()!=2 || other.size()!=0) vetoEvent;
      if(DD[0].pid()!=-DD[1].pid()) vetoEvent;
      FourMomentum pDD = DD[0].momentum()+DD[1].momentum();
      // cross section
      if(DD[0].parents()[0].pid()==_pid && DD[1].parents()[0].pid()==_pid) {
	_c->fill();
      }
      // histograms only at 10.58GeV
      if(!_h[0]) return;
      _h[0]->fill(pDD.mass());
      double cProd = axis.dot(JPSI[0].momentum().p3().unit());
      _h[1]->fill(cProd);
      // finally the leptons from J/psi decay
      if(JPSI[0].children().size()!=2) vetoEvent;
      if(JPSI[0].children()[0].pid()!=-JPSI[0].children()[1].pid()) vetoEvent;
      if(JPSI[0].children()[0].abspid()!=PID::EMINUS &&
	 JPSI[0].children()[0].abspid()!=PID::MUON) vetoEvent;
      Particle lm = JPSI[0].children()[0];
      Particle lp = JPSI[0].children()[1];
      // variables in J/psi frame
      Vector3 axis2 = -pDD.p3().unit();
      LorentzTransform boost1 = LorentzTransform::mkFrameTransformFromBeta(JPSI[0].momentum().betaVec());
      FourMomentum plm = boost1.transform(lm.momentum());
      Vector3 axis4 = plm.p3().unit();
      double cPsi = axis2.dot(axis4);
      _h[2]->fill(cPsi);
      Vector3 axis3 = boost1.transform(pep).p3().unit();
      Vector3 aep = axis3-axis3.dot(axis2)*axis2;
      Vector3 alm = axis4-axis4.dot(axis2)*axis2;
      double phil = atan2(-axis2.cross(aep).dot(alm),aep.dot(alm));
      _h[4]->fill(phil);
      // variables in DD frame
      LorentzTransform boost2 = LorentzTransform::mkFrameTransformFromBeta(pDD.betaVec());
      FourMomentum pD = boost2.transform((DD[0].pid()>0 ? DD[0] : DD[1]).momentum());
      Vector3 axisD = pD.p3().unit();
      axis2 *= -1;
      double cX = axis2.dot(axisD);
      _h[3]->fill(cX);
      axis3 = boost2.transform(pep).p3().unit();
      aep = axis3-axis3.dot(axis2)*axis2;
      alm = axisD-axisD.dot(axis2)*axis2;
      phil = atan2(-axis2.cross(aep).dot(alm),aep.dot(alm));
      _h[5]->fill(phil);
    }


    /// Normalise histograms etc., after the run
    void finalize() {
      if(_h[0]) {
	for(unsigned int ix=0;ix<6;++ix) {
	  normalize(_h[ix],1.,false);
	}
      }
      // cross sections
      double fact = crossSection()/ sumOfWeights() /femtobarn;
      double sigma = _c->val()*fact;
      double error = _c->err()*fact;
      Scatter2D temphisto(refData(2, 1, 1));
      Scatter2DPtr  mult;
      book(mult, 2, 1, 1);
      for (size_t b = 0; b < temphisto.numPoints(); b++) {
	const double x  = temphisto.point(b).x();
	pair<double,double> ex = temphisto.point(b).xErrs();
	pair<double,double> ex2 = ex;
	if(ex2.first ==0.) ex2. first=0.001;
	if(ex2.second==0.) ex2.second=0.001;
	if (inRange(sqrtS()/GeV, x-ex2.first, x+ex2.second)) {
	  mult->addPoint(x, sigma, ex, make_pair(error,error));
	}
	else {
	  mult->addPoint(x, 0., ex, make_pair(0.,.0));
	}
      }
    }

    /// @}


    /// @name Histograms
    /// @{
    int _pid;
    Histo1DPtr _h[6];
    CounterPtr _c;
    /// @}

  };


  RIVET_DECLARE_PLUGIN(BELLE_2017_I1590028);

}
